<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwzip="urn:WebWorks-XSLT-Extension-Zip" 
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwunits="urn:WebWorks-XSLT-Extension-Units"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              exclude-result-prefixes="xsl msxsl wwmode wwfiles wwzip wwdoc wwsplits wwproject wwprogress wwlog wwfilesystem wwuri wwstring wwunits wwfilesext wwprojext wwexsldoc"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterDependsType" />
 <xsl:param name="ParameterMergeDependsType" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterMergeType" />
 <xsl:param name="ParameterSplitsType" />
 <xsl:param name="ParameterJarFilesDeployAttribute" />
 <xsl:param name="ParameterExecutableJarSplitFileType" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />
 <xsl:param name="ParameterMergeCategory" />
 <xsl:param name="ParameterMergeUse" />
 <xsl:param name="ParameterMergeDeploy" />


 <xsl:output method="xml" encoding="UTF-8" indent="yes" />
 <xsl:strip-space elements="*" />

 <xsl:key name="wwfiles-files-by-type" match="wwfiles:File" use="@type" />
 <xsl:key name="wwfiles-files-by-groupid-type" match="wwfiles:File" use="concat(@groupID, ':', @type)" />
 <xsl:key name="wwfiles-files-by-deploy" match="wwfiles:File" use="@deploy" />
 <xsl:key name="wwfiles-files-by-groupid-deploy" match="wwfiles:File" use="concat(@groupID, ':', @deploy)" />
 <xsl:key name="wwsplits-files-by-groupid-type" match="wwsplits:File" use="concat(@groupID, ':', @type)" />


 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <xsl:value-of select="wwprogress:Start(2)" />

   <xsl:value-of select="wwprogress:Start(1)" />
   <!-- Iterate input documents -->
   <!--                         -->
   <xsl:for-each select="$GlobalFiles[1]">
    <xsl:variable name="VarManifestFiles" select="key('wwfiles-files-by-type', $ParameterDependsType)" />

    <xsl:value-of select="wwprogress:Start(count($VarManifestFiles))" />

    <xsl:for-each select="$VarManifestFiles">
     <xsl:variable name="VarManifestFile" select="." />

     <xsl:value-of select="wwprogress:Start(1)" />

     <!-- Aborted? -->
     <!--          -->
     <xsl:if test="not(wwprogress:Abort())">
      <!-- Load splits -->
      <!--             -->
      <xsl:variable name="VarSplitsFile" select="key('wwfiles-files-by-groupid-type', concat($VarManifestFile/@groupID, ':', $ParameterSplitsType))[1]" />
      <xsl:variable name="VarSplits" select="wwexsldoc:LoadXMLWithoutResolver($VarSplitsFile/@path)" />

      <!-- Determine group name -->
      <!--                      -->
      <xsl:variable name="VarGroupName" select="wwprojext:GetGroupName($VarManifestFile/@groupID)" />

      <!-- Jar Path -->
      <!--          -->
      <xsl:variable name="VarJarPath">
       <xsl:for-each select="$VarSplits[1]">
        <xsl:value-of select="key('wwsplits-files-by-groupid-type', concat($VarManifestFile/@groupID, ':', $ParameterExecutableJarSplitFileType))[1]/@path" />
       </xsl:for-each>
      </xsl:variable>

      <!-- Jar Files -->
      <!--           -->
      <xsl:for-each select="$GlobalFiles[1]">
       <xsl:variable name="VarJarFiles" select="key('wwfiles-files-by-groupid-deploy', concat($VarManifestFile/@groupID, ':', $ParameterJarFilesDeployAttribute))" />
       <xsl:variable name="VarJarFilesChecksum">
        <xsl:call-template name="FilesChecksum">
         <xsl:with-param name="ParamFiles" select="$VarJarFiles" />
        </xsl:call-template>
       </xsl:variable>

       <!-- Call template -->
       <!--               -->
       <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarJarPath, concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarJarFilesChecksum), $VarManifestFile/@groupID, '', $GlobalActionChecksum)" />
       <xsl:if test="not($VarUpToDate)">
        <xsl:variable name="VarResult">
         <xsl:call-template name="Jar">
          <xsl:with-param name="ParamReferenceDirectoryPath" select="wwfilesystem:GetDirectoryName($VarManifestFile/@path)" />
          <xsl:with-param name="ParamManifestPath" select="$VarManifestFile/@path" />
          <xsl:with-param name="ParamJarPath" select="$VarJarPath" />
          <xsl:with-param name="ParamJarFiles" select="$VarJarFiles" />
         </xsl:call-template>
        </xsl:variable>
       </xsl:if>

       <!-- Aborted? -->
       <!--          -->
       <xsl:if test="not(wwprogress:Abort())">
        <wwfiles:File path="{$VarJarPath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($VarJarPath)}" projectchecksum="{concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarJarFilesChecksum)}" groupID="{$VarManifestFile/@groupID}" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
         <wwfiles:Depends path="{$VarManifestFile/@path}" checksum="{wwfilesystem:GetChecksum($VarManifestFile/@path)}" groupID="{$VarManifestFile/@groupID}" documentID="{$VarManifestFile/@documentID}" />

         <xsl:call-template name="FilesAsDepends">
          <xsl:with-param name="ParamFiles" select="$VarJarFiles" />
         </xsl:call-template>
        </wwfiles:File>
       </xsl:if>
      </xsl:for-each>
     </xsl:if>

     <xsl:value-of select="wwprogress:End()" />
    </xsl:for-each>

    <xsl:value-of select="wwprogress:End()" />
   </xsl:for-each>
   <xsl:value-of select="wwprogress:End()" />

   <!-- Merge -->
   <!--       -->
   <xsl:value-of select="wwprogress:Start(1)" />
   <xsl:variable name="VarProjectGroups" select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group" />
   <xsl:if test="count($VarProjectGroups) &gt; 1">
    <xsl:for-each select="$GlobalFiles[1]">
     <xsl:variable name="VarMergeHelpSetFile" select="key('wwfiles-files-by-type', $ParameterMergeDependsType)[1]" />
     <xsl:if test="count($VarMergeHelpSetFile) = 1">
      <xsl:variable name="VarMergePath" select="wwfilesystem:GetWithExtensionReplaced($VarMergeHelpSetFile/@path, '.jar')" />

      <!-- Aborted? -->
      <!--          -->
      <xsl:if test="not(wwprogress:Abort())">
       <!-- Jar Files -->
       <!--           -->
       <xsl:for-each select="$GlobalFiles[1]">
        <xsl:variable name="VarJarFiles" select="key('wwfiles-files-by-deploy', $ParameterJarFilesDeployAttribute)" />
        <xsl:variable name="VarJarFilesChecksum">
         <xsl:call-template name="FilesChecksum">
          <xsl:with-param name="ParamFiles" select="$VarJarFiles" />
         </xsl:call-template>
        </xsl:variable>

        <!-- Call template -->
        <!--               -->
        <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($VarMergePath, concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarJarFilesChecksum), '', '', $GlobalActionChecksum)" />
        <xsl:if test="not($VarUpToDate)">
         <xsl:variable name="VarResult">
          <xsl:call-template name="Jar">
           <xsl:with-param name="ParamExecutionDirectory" select="wwfilesystem:GetDirectoryName($VarMergeHelpSetFile/@path)" />
           <xsl:with-param name="ParamManifestPath" select="$VarMergeHelpSetFile/@path" />
           <xsl:with-param name="ParamJarPath" select="$VarMergePath" />
           <xsl:with-param name="ParamJarFiles" select="$VarJarFiles" />
          </xsl:call-template>
         </xsl:variable>
        </xsl:if>

        <!-- Aborted? -->
        <!--          -->
        <xsl:if test="not(wwprogress:Abort())">
         <wwfiles:File path="{$VarMergePath}" type="{$ParameterMergeType}" checksum="{wwfilesystem:GetChecksum($VarMergePath)}" projectchecksum="{concat($GlobalProject/wwproject:Project/@ChangeID, ':', $VarJarFilesChecksum)}" groupID="" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterMergeCategory}" use="{$ParameterMergeUse}" deploy="{$ParameterMergeDeploy}">
          <wwfiles:Depends path="{$VarMergeHelpSetFile/@path}" checksum="{$VarMergeHelpSetFile/@checksum}" groupID="{$VarMergeHelpSetFile/@groupID}" documentID="{$VarMergeHelpSetFile/@documentID}" />

          <xsl:call-template name="FilesAsDepends">
           <xsl:with-param name="ParamFiles" select="$VarJarFiles" />
          </xsl:call-template>
         </wwfiles:File>
        </xsl:if>
       </xsl:for-each>
      </xsl:if>
     </xsl:if>
    </xsl:for-each>
   </xsl:if>
   <xsl:value-of select="wwprogress:End()" />

   <xsl:value-of select="wwprogress:End()" />

  </wwfiles:Files>
 </xsl:template>


 <xsl:template name="FilesChecksum">
  <xsl:param name="ParamFiles" />

  <xsl:variable name="VarFilePaths">
   <xsl:for-each select="$ParamFiles">
    <xsl:variable name="VarFile" select="." />

    <xsl:value-of select="$VarFile/@path" />
    <xsl:text>:</xsl:text>
   </xsl:for-each>
  </xsl:variable>

  <xsl:value-of select="wwstring:MD5Checksum($VarFilePaths)" />
 </xsl:template>


 <xsl:template name="FilesAsDepends">
  <xsl:param name="ParamFiles" />

  <xsl:for-each select="$ParamFiles">
   <xsl:variable name="VarFile" select="." />

   <wwfiles:Depends path="{$VarFile/@path}" checksum="{$VarFile/@checksum}" groupID="{$VarFile/@groupID}" documentID="{$VarFile/@documentID}" />
  </xsl:for-each>
 </xsl:template>


 <xsl:template name="Jar">
  <xsl:param name="ParamReferenceDirectoryPath" />
  <xsl:param name="ParamManifestPath" />
  <xsl:param name="ParamJarPath" />
  <xsl:param name="ParamJarFiles" />

  <!-- Create archive starting with manifest -->
  <!--                                       -->
  <xsl:variable name="VarTempManifestPath" select="wwfilesystem:Combine(wwfilesystem:GetTempPath(), 'MANIFEST.MF')" />
  <xsl:variable name="VarCopyManifest" select="wwfilesystem:CopyFile($ParamManifestPath, $VarTempManifestPath)" />
  <xsl:variable name="VarManifestFileListAsXML">
   <wwzip:File source="{$VarTempManifestPath}" zip-directory="META-INF" />
  </xsl:variable>
  <xsl:variable name="VarManifestFileList" select="msxsl:node-set($VarManifestFileListAsXML)" />
  <xsl:variable name="VarJarManifestFileList" select="wwzip:Zip($ParamJarPath, $VarManifestFileList)" />
  <xsl:variable name="VarDeleteTempManifest" select="wwfilesystem:DeleteFile($VarTempManifestPath)" />

  <!-- Add files -->
  <!--           -->
  <xsl:variable name="VarFilesToAddListAsXML">
   <!-- Reference path for get relative path calls -->
   <!--                                            -->
   <xsl:variable name="VarReferencePath" select="wwfilesystem:Combine($ParamReferenceDirectoryPath, 'dummy.component')" />

   <!-- Iterate file list -->
   <!--                   -->
   <xsl:for-each select="$ParamJarFiles">
    <xsl:variable name="VarJarFile" select="." />

    <xsl:variable name="VarJarDirectory" select="wwfilesystem:GetDirectoryName(wwfilesystem:GetRelativeTo($VarJarFile/@path, $VarReferencePath))" />
    <wwzip:File source="{$VarJarFile/@path}" zip-directory="{$VarJarDirectory}" />
   </xsl:for-each>
  </xsl:variable>
  <xsl:variable name="VarFilesToAddList" select="msxsl:node-set($VarFilesToAddListAsXML)" />
  <xsl:variable name="VarJarFilesToAddList" select="wwzip:ZipAdd($ParamJarPath, $VarFilesToAddList)" />
 </xsl:template>
</xsl:stylesheet>
