<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                              xmlns:msxsl="urn:schemas-microsoft-com:xslt"
                              xmlns:wwmode="urn:WebWorks-Engine-Mode"
                              xmlns:wwfiles="urn:WebWorks-Engine-Files-Schema"
                              xmlns:wwdoc="urn:WebWorks-Document-Schema"
                              xmlns:wwsplits="urn:WebWorks-Engine-Splits-Schema"
                              xmlns:wwproject="urn:WebWorks-Publish-Project"
                              xmlns:wwprogress="urn:WebWorks-XSLT-Extension-Progress"
                              xmlns:wwlog="urn:WebWorks-XSLT-Extension-Log"
                              xmlns:wwfilesystem="urn:WebWorks-XSLT-Extension-FileSystem"
                              xmlns:wwuri="urn:WebWorks-XSLT-Extension-URI"
                              xmlns:wwstring="urn:WebWorks-XSLT-Extension-StringUtilities"
                              xmlns:wwunits="urn:WebWorks-XSLT-Extension-Units"
                              xmlns:wwfilesext="urn:WebWorks-XSLT-Extension-Files"
                              xmlns:wwprojext="urn:WebWorks-XSLT-Extension-Project"
                              xmlns:wwexsldoc="urn:WebWorks-XSLT-Extension-Document"
                              xmlns:wwexec="urn:WebWorks-XSLT-Extension-Execute"
                              xmlns:wwenv="urn:WebWorks-XSLT-Extension-Environment"
                              exclude-result-prefixes="xsl msxsl wwmode wwfiles wwdoc wwsplits wwproject wwprogress wwlog wwfilesystem wwuri wwstring wwunits wwfilesext wwprojext wwexsldoc wwexec wwenv"
>
 <xsl:param name="GlobalInput" />
 <xsl:param name="GlobalPipelineName" />
 <xsl:param name="GlobalProject" />
 <xsl:param name="GlobalFiles" />
 <xsl:param name="ParameterType" />
 <xsl:param name="ParameterJavaHelpURI" />
 <xsl:param name="ParameterCategory" />
 <xsl:param name="ParameterUse" />
 <xsl:param name="ParameterDeploy" />


 <xsl:output method="xml" encoding="UTF-8" indent="yes" />
 <xsl:strip-space elements="*" />

 <xsl:variable name="GlobalActionChecksum">
  <xsl:variable name="VarTransformChecksums">
   <xsl:value-of select="concat(wwuri:AsFilePath('wwtransform:self'), ':', wwfilesystem:GetChecksum(wwuri:AsFilePath('wwtransform:self')))" />
  </xsl:variable>
  <xsl:value-of select="wwstring:MD5Checksum($VarTransformChecksums)" />
 </xsl:variable>


 <xsl:template match="/">
  <wwfiles:Files version="1.0">

   <!-- Set JavaHelp path -->
   <!--                   -->
   <xsl:variable name="VarJavaHelpPath" select="wwprojext:GetFormatSetting('local-javahelp-location', wwuri:AsFilePath($ParameterJavaHelpURI))" />

   <!-- At least one group defined? -->
   <!--                             -->
   <xsl:for-each select="$GlobalProject/wwproject:Project/wwproject:Groups/wwproject:Group[1]">
    <!-- Runtime Jars -->
    <!--              -->
    <xsl:call-template name="CopyRuntimeFile">
     <xsl:with-param name="ParamSourceFilePath" select="wwuri:AsFilePath('wwhelper:Libraries/java/runjar.jar')" />
     <xsl:with-param name="ParamDestinationFilePath" select="wwfilesystem:Combine(wwprojext:GetTargetOutputDirectoryPath(), 'runjar.jar')" />
    </xsl:call-template>

    <xsl:call-template name="CopyRuntimeFile">
     <xsl:with-param name="ParamSourceFilePath" select="wwfilesystem:Combine($VarJavaHelpPath, 'javahelp', 'lib', 'jh.jar')" />
     <xsl:with-param name="ParamDestinationFilePath" select="wwfilesystem:Combine(wwprojext:GetTargetOutputDirectoryPath(), 'jh.jar')" />
    </xsl:call-template>

    <xsl:call-template name="CopyRuntimeFile">
     <xsl:with-param name="ParamSourceFilePath" select="wwfilesystem:Combine($VarJavaHelpPath, 'demos', 'bin', 'hsviewer.jar')" />
     <xsl:with-param name="ParamDestinationFilePath" select="wwfilesystem:Combine(wwprojext:GetTargetOutputDirectoryPath(), 'hsviewer.jar')" />
    </xsl:call-template>
   </xsl:for-each>

  </wwfiles:Files>
 </xsl:template>


 <xsl:template name="CopyRuntimeFile">
  <xsl:param name="ParamSourceFilePath" />
  <xsl:param name="ParamDestinationFilePath" />

  <xsl:variable name="VarUpToDate" select="wwfilesext:UpToDate($ParamDestinationFilePath, $ParamSourceFilePath, '', '', $GlobalActionChecksum)" />
  <xsl:if test="not($VarUpToDate)">
   <xsl:variable name="VarIgnore">
    <xsl:value-of select="wwfilesystem:CopyFile($ParamSourceFilePath, $ParamDestinationFilePath)" />
   </xsl:variable>
  </xsl:if>
  <wwfiles:File path="{$ParamDestinationFilePath}" type="{$ParameterType}" checksum="{wwfilesystem:GetChecksum($ParamDestinationFilePath)}" projectchecksum="" groupID="" documentID="" actionchecksum="{$GlobalActionChecksum}" category="{$ParameterCategory}" use="{$ParameterUse}" deploy="{$ParameterDeploy}">
   <wwfiles:Depends path="{$ParamSourceFilePath}" checksum="{wwfilesystem:GetChecksum($ParamSourceFilePath)}" groupID="" documentID="" />
  </wwfiles:File>
 </xsl:template>
</xsl:stylesheet>
